using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Toolkit.Forms {
	/// <summary>
	/// Xg{bNX̃hbOhbv̏sB
	/// </summary>
	/// <typeparam name="ItemType">Xg{bNXɊ܂܂ACě^</typeparam>
	public static class ListBoxDragDrop<ItemType> {
		/// <summary>
		/// hbOhbvɂ郊Xg{bNXł̍ڂ̕ёւ\ɂ܂B
		/// </summary>
		/// <param name="listBox">ΏۂƂȂ郊Xg{bNX</param>
		/// <remarks>
		/// DoDragDrop̂͌ĂяoŁB
		/// DragDropEffectsCopyMoveɂΉB
		/// B
		/// <code>
		/// Point? dragBeginLocation = null;
		///
		/// private void listBox1_MouseDown(object sender, MouseEventArgs exception) {
		/// 	if (e.Button == MouseButtons.Left && e.Clicks == 1) {
		/// 		dragBeginLocation = e.Location;
		/// 	}
		/// }
		///
		/// private void listBox1_MouseMove(object sender, MouseEventArgs exception) {
		/// 	if (dragBeginLocation.HasValue && dragBeginLocation.Value != e.Location) {
		/// 		dragBeginLocation = null;
		/// 		object item = listBox1.SelectedItem;
		/// 		if (item != null) {
		/// 			listBox1.DoDragDrop(item, DragDropEffects.Move | DragDropEffects.Copy);
		/// 		}
		/// 	}
		/// }
		///
		/// private void listBox1_MouseUp(object sender, MouseEventArgs exception) {
		/// 	dragBeginLocation = null;
		/// }
		/// </code>
		/// </remarks>
		public static void Register(ListBox listBox) {
			System.Diagnostics.Debug.Assert(listBox.SelectionMode == SelectionMode.One);
			listBox.AllowDrop = true;
			listBox.DragEnter += new DragEventHandler(listBox_DragEnter);
			listBox.DragOver += new DragEventHandler(listBox_DragOver);
			listBox.DragDrop += new DragEventHandler(listBox_DragDrop);
		}

		static void listBox_DragEnter(object sender, DragEventArgs e) {
			if (!e.Data.GetDataPresent(typeof(ItemType))) return;
			// GtFNg̎w
			Utility.SetDropEffect(e, DragDropEffects.Move);
		}

		static void listBox_DragOver(object sender, DragEventArgs e) {
			if (!e.Data.GetDataPresent(typeof(ItemType))) return;
			ListBox control = (ListBox)sender;

			object dstItem = Utility.GetDropTarget(control, e);
			if (dstItem != null) {
				control.SelectedItem = dstItem;
			}
			// GtFNg̎w
			Utility.SetDropEffect(e, DragDropEffects.Move);
		}

		static void listBox_DragDrop(object sender, DragEventArgs e) {
			if (!e.Data.GetDataPresent(typeof(ItemType))) return;
			ListBox control = (ListBox)sender;

			ItemType srcItem = (ItemType)e.Data.GetData(typeof(ItemType));
			if (srcItem == null) return;

			int? dstItemIndex_ = Utility.GetDropTargetIndex(control, e);
			int dstItemIndex = dstItemIndex_.HasValue ? dstItemIndex_.Value : control.Items.Count - 1;

			if (e.Effect == DragDropEffects.Move) { // move
				control.Items.Remove(srcItem);
				control.Items.Insert(dstItemIndex, srcItem);
			} else { // copy
				ICloneable cloneable = srcItem as ICloneable;
				if (cloneable == null) {
					System.Diagnostics.Debug.Fail("Xg{bNX̃ACeICloneableł͂܂");
					control.Items.Insert(dstItemIndex, srcItem);
				} else {
					control.Items.Insert(dstItemIndex, cloneable.Clone());
				}
			}
			control.SelectedIndex = dstItemIndex;
		}
	}
}
