using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace Toolkit.Forms {
	/// <summary>
	/// Xibv
	/// </summary>
	public interface ISnapTarget {
		Rectangle Rect { get; }

		/// <summary>
		/// DxBꂪقǗDIɋz
		/// </summary>
		int Priority { get; }
	}

	/// <summary>
	/// ScreenȃXibv^[Qbg
	/// </summary>
	public class SnapTargetScreen : ISnapTarget {
		private Screen screen;
		public Screen Screen { get { return screen; } }

		public SnapTargetScreen(Screen screen) {
			this.screen = screen;
		}

		/// <param name="n">Ԗڂ̃XN[ĝB</param>
		public SnapTargetScreen(int n) {
			this.screen = Screen.AllScreens[n];
		}

		#region ISnapTarget o

		public Rectangle Rect {
			get {
				return screen.WorkingArea;
			}
		}

		public int Priority {
			get { return 2000; }
		}

		#endregion
	}

	/// <summary>
	/// Form, ControlȃXibv^[Qbg
	/// </summary>
	public class SnapTargetControl : ISnapTarget {
		private Control control;
		public Control Control { get { return control; } }

		public SnapTargetControl(Control control) {
			this.control = control;
		}

		#region ISnapTarget o

		public Rectangle Rect {
			get {
				return new Rectangle(control.Location, control.Size);
			}
		}

		public int Priority {
			get { return 1000; }
		}

		#endregion
	}

	/// <summary>
	/// tH[ʒ[Ȃǂɋz
	/// </summary>
	public class Snapper : Component {
		bool enable = true;
		int snapDistance = 24;
		List<ISnapTarget> targets = new List<ISnapTarget>();

		System.Windows.Forms.Form form;
		SubclassWrapper subclassWrapper;

		bool recursionLock = false;
		bool dragging;
		Size dragOffset; // Cursor.Position - form.Location;

		public Snapper() {
			// ftHgőSXN[^[QbgƂĂB
			// XN[̌͓Iɕς邱Ƃ͖őɖƎv̂ŁBB
			foreach (Screen s in Screen.AllScreens) {
				targets.Add(new SnapTargetScreen(s));
			}
		}

		#region ݒƂ

		/// <summary>
		/// L
		/// </summary>
		[Category("Snapper")]
		[Description("zsǂݒ肵܂")]
		[DefaultValue(true)]
		public bool Enable {
			get { return enable; }
			set { enable = value; }
		}

		/// <summary>
		/// Xibv鋗
		/// </summary>
		[Category("Snapper")]
		[Description("zsݒ肵܂")]
		[DefaultValue(24)]
		public int SnapDistance {
			get { return snapDistance; }
			set { snapDistance = value; }
		}

		#endregion

		#region publicȃC^[tF[X

		/// <summary>
		/// z^[Qbg̃Xg
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public List<ISnapTarget> Targets {
			get { return targets; }
		}

		/// <summary>
		/// z鑤o^
		/// </summary>
		public void Register(Form form) {
			// Cxg̓o^
			this.form = form;
			this.form.ResizeBegin += new EventHandler(form_ResizeBegin);
			this.form.ResizeEnd += new EventHandler(form_ResizeEnd);
			subclassWrapper = new SubclassWrapper(this.form);
			subclassWrapper.PostWndProc += new EventHandler<WndProcEventArgs>(subclassWrapper_PostWndProc);
		}

		/// <summary>
		/// zȂ烊TCYƂړƂ
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Rectangle ClientRectangle {
			get { return form.ClientRectangle; }
			set {
				Rectangle rc = form.RectangleToScreen(value);
				SnapRect(ref rc, new Rectangle(form.Location, form.Size));
				form.Size = rc.Size;
				Utility.SetLocationWithClip(form, rc.Location);
			}
		}

		/// <summary>
		/// zȂ烊TCY
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Size ClientSize {
			get { return form.ClientSize; }
			set {
				Rectangle rc = form.RectangleToScreen(new Rectangle(new Point(0, 0), value));
				SnapRect(ref rc, new Rectangle(form.Location, form.Size));
				form.Size = rc.Size;
				Utility.SetLocationWithClip(form, rc.Location);
			}
		}

		/// <summary>
		/// zȂړƂ
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Point Location {
			get { return form.Location; }
			set {
				Rectangle rc = new Rectangle(value, form.Size);
				SnapRect(ref rc, new Rectangle(form.Location, form.Size));
				Utility.SetLocationWithClip(form, rc.Location);
				System.Diagnostics.Debug.Assert(rc.Size == form.Size);
			}
		}

		/// <summary>
		/// zȂ烊TCY
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public Size Size {
			get { return form.Size; }
			set {
				Rectangle rc = new Rectangle(form.Location, value);
				SnapRect(ref rc, new Rectangle(form.Location, form.Size));
				form.Size = rc.Size;
				System.Diagnostics.Debug.Assert(rc.Size == value);
			}
		}

		#endregion

		#region ƂB

		void form_ResizeBegin(object sender, EventArgs e) {
			lock (targets) {
				if (!enable) return;
				// hbOĂItZbgL^
				dragOffset.Width = Cursor.Position.X - form.Location.X;
				dragOffset.Height = Cursor.Position.Y - form.Location.Y;
				dragging = true;
			}
		}

		void form_ResizeEnd(object sender, EventArgs e) {
			lock (targets) {
				dragging = false;
			}
		}

		[StructLayout(LayoutKind.Sequential)]
		struct RECT
		{
			public int left;
			public int top;
			public int right;
			public int bottom;
		}

		void subclassWrapper_PostWndProc(object sender, WndProcEventArgs e) {
			const int WM_MOVING = 0x0216;
			if (e.Message.Msg == WM_MOVING) {
				RECT r = (RECT)Marshal.PtrToStructure(e.Message.LParam, typeof(RECT));
				Rectangle rc = new Rectangle(r.left, r.top, r.right - r.left, r.bottom - r.top);
				form_Moving(ref rc);
				r.left = rc.Left;
				r.top = rc.Top;
				r.right = rc.Right;
				r.bottom = rc.Bottom;
				Marshal.StructureToPtr(r, e.Message.LParam, false);
			}
		}

		void form_Moving(ref Rectangle rc) {
			lock (targets) {
				if (!enable) return;
				if (!dragging) return;
				if (!recursionLock) {
					try {
						recursionLock = true;

						System.Diagnostics.Debug.Assert(rc.Size == form.Size);
						SnapRect(ref rc, new Rectangle(Cursor.Position - dragOffset, rc.Size));

					} finally {
						recursionLock = false;
					}
				}
			}
		}

		/// <summary>
		/// zRectԂB
		/// </summary>
		void SnapRect(ref Rectangle newRect, Rectangle nowRect) {
			// zĂ^[QbgƕT
			ISnapTarget target = null;
			bool[] snap = null;
			int distCount = 0;
			foreach (ISnapTarget t in targets) {
				if (SnapTargetIsOwn(t)) continue;
				// zׂ
				bool[] s = new bool[8]{
					IsInV(nowRect, t.Rect) && Math.Abs(nowRect.Left - t.Rect.Left) <= snapDistance,
					IsInV(nowRect, t.Rect) && Math.Abs(nowRect.Left - t.Rect.Right) <= snapDistance,
					IsInH(nowRect, t.Rect) && Math.Abs(nowRect.Top - t.Rect.Top) <= snapDistance,
					IsInH(nowRect, t.Rect) && Math.Abs(nowRect.Top - t.Rect.Bottom) <= snapDistance,
					IsInV(nowRect, t.Rect) && Math.Abs(nowRect.Right - t.Rect.Right) <= snapDistance,
					IsInV(nowRect, t.Rect) && Math.Abs(nowRect.Right - t.Rect.Left) <= snapDistance,
					IsInH(nowRect, t.Rect) && Math.Abs(nowRect.Bottom - t.Rect.Bottom) <= snapDistance,
					IsInH(nowRect, t.Rect) && Math.Abs(nowRect.Bottom - t.Rect.Top) <= snapDistance};
				int count = Array.FindAll<bool>(s, delegate(bool b) { return b; }).Length;
				if (0 < count) {
					if (target == null || t.Priority * distCount < t.Priority * count) {
						target = t;
						snap = s;
						distCount = count;
					}
				}
			}
			// zΏۂȂ牽Ȃ
			if (target == null) {
				return;
			}
			// z
			if (snap[0]) { // left:left
				newRect.X = target.Rect.Left;
			} else if (snap[1]) { // left:right
				newRect.X = target.Rect.Right;
			} else if (snap[4]) { // right:right
				newRect.X = target.Rect.Right - newRect.Width;
			} else if (snap[5]) { // right:left
				newRect.X = target.Rect.Left - newRect.Width;
			}
			if (snap[2]) { // top:top
				newRect.Y = target.Rect.Top;
			} else if (snap[3]) { // top:bottom
				newRect.Y = target.Rect.Bottom;
			} else if (snap[6]) { // bottom:bottom
				newRect.Y = target.Rect.Bottom - newRect.Height;
			} else if (snap[7]) { // bottom:top
				newRect.Y = target.Rect.Top - newRect.Height;
			}
		}

		private bool SnapTargetIsOwn(ISnapTarget t) {
			SnapTargetControl stc = t as SnapTargetControl;
			return stc != null && stc.Control == form;
		}

		bool IsInH(Rectangle x, Rectangle y) {
			return y.Left <= x.Right && x.Left <= y.Right;
		}
		bool IsInV(Rectangle x, Rectangle y) {
			return y.Top <= x.Bottom && x.Top <= y.Bottom;
		}

		#endregion
	}
}
