using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace Toolkit.Net.Mail {
	/// <summary>
	/// IMAP
	/// </summary>
	public class IMAP : IProtocol {
		string clientName = "MyIMAP"; // ꉞ₷OɁB딚Ă̂ŁBB
		bool useLIST = false;
		bool useLSUB = false;
		bool checkUnseen = false; // ǂVɂȂtrue
		string currentDir = null;
		List<string> dirs = null;
		List<MailEntry> mails = null;

		struct MailEntry {
			public string Directory;
			public string UID;
			public MailEntry(string directory, string uid) {
				Directory = directory;
				UID = uid;
			}
		}

		public string ClientName {
			get { return clientName; }
			set { clientName = value; }
		}

		public bool UseLIST {
			get { return useLIST; }
			set { useLIST = value; }
		}

		public bool UseLSUB {
			get { return useLSUB; }
			set { useLSUB = value; }
		}

		public bool CheckUnseen {
			get { return checkUnseen; }
			set { checkUnseen = value; }
		}

		#region IMailReceiveProtocol o

		public int DefaultPort(bool ssl) {
			return ssl ? 993 : 143;
		}

		public bool Login(Stream stream, string user, string pass) {
			try {
				string str = Utility.Recv(stream);
				System.Diagnostics.Debug.Assert(str.Contains("* OK "));
				// F
				Utility.Send(stream, clientName + " AUTHENTICATE CRAM-MD5\r\n");
				str = Utility.Recv(stream);
				Match m = Regex.Match(str, "^\\+ (.+?)\\s*$");
				if (m.Success) {
					byte[] challenge = Convert.FromBase64String(m.Groups[1].Value);
					System.Diagnostics.Debug.WriteLine("CRAM-MD5 Challenge: " + Encoding.Default.GetString(challenge));
					string md5 = Hash.HMACMD5(Encoding.Default.GetBytes(pass), challenge);
					Utility.Send(stream, Convert.ToBase64String(Encoding.Default.GetBytes(user + " " + md5)) + "\r\n");
					str = Receive(stream);
					//if (str.Contains(clientName + " OK")) {
					return true;
				}
			} catch (Exception e) {
				System.Diagnostics.Debug.Fail(e.Message);
			}
			return false;
		}

		public int GetMailCount(Stream stream) {
			// Ɏ擾ς݂Ȃ炻Ԃ
			if (mails != null) {
				return mails.Count;
			}
#if true
			// fBNg̃XgAbv
			ListupDirs(stream);
			// efBNgV̎擾
			List<MailEntry> mailsTmp = new List<MailEntry>();
			foreach (string dir in dirs) {
				SearchUIDs(stream, mailsTmp, dir);
			}
#else
			List<string> mailsTmp = new List<string>();
			SearchUIDs(stream, mailsTmp, "INBOX");
#endif
			mails = Utility.Unique(mailsTmp);
			return mails.Count;
		}

		private void SearchUIDs(Stream stream, List<MailEntry> mailsTmp, string dir) {
			Examine(stream, dir);
			// 擾
			//List<string> uids = InnerSearch(stream, "UNDELETED OR RECENT UNSEEN"); // (&& UNDELETED (|| RECENT UNSEEN))
			//List<string> uids = InnerSearch(stream, "UNSEEN UNDELETED");
			List<string> uids =  InnerSearch(stream, checkUnseen ?
				"UNDELETED OR RECENT UNSEEN" :
				"UNDELETED RECENT UNSEEN");
			mailsTmp.AddRange(uids.ConvertAll(uid => new MailEntry(dir, uid)));
			// UIDɂ̓fBNgĂB
		}

		public string ReceiveHeader(Stream stream, int n) {
			Examine(stream, mails[n].Directory);
			Utility.Send(stream, clientName + " UID FETCH " + mails[n].UID + " (RFC822.HEADER)\r\n");
			return Receive(stream);
		}

		public string Receive(Stream stream, int n) {
			Examine(stream, mails[n].Directory);
			Utility.Send(stream, clientName + " UID FETCH " + mails[n].UID + " (RFC822.HEADER, BODY)\r\n");
			return Receive(stream);
		}

		public string GetUniqueID(Stream stream, int n) {
			return mails[n].Directory + "\0" + mails[n].UID;
		}

		public void Logout(Stream stream) {
			Utility.Send(stream, clientName + " LOGOUT\r\n");
			currentDir = null;
			dirs = null;
			mails = null;
		}

		#endregion

		/// <summary>
		/// fBNg̃XgAbv
		/// </summary>
		private void ListupDirs(Stream stream) {
			if (dirs == null) {
				List<string> dirsTmp = new List<string>();
				if (useLIST) {
					Utility.Send(stream, clientName + " LIST \"\" \"*\"\r\n");
					ReceiveLIST(stream, "LIST", dirsTmp);
				}
				if (useLSUB) {
					Utility.Send(stream, clientName + " LSUB \"\" \"*\"\r\n");
					ReceiveLIST(stream, "LSUB", dirsTmp);
				}
				if (!useLIST && !useLSUB) {
					System.Diagnostics.Debug.Assert(dirsTmp.Count == 0);
					dirsTmp.Add("INBOX");
				}
				dirs = Utility.Unique(dirsTmp);
			}
		}

		private void ReceiveLIST(Stream stream, string op, List<string> dirsTmp) {
			string str = Receive(stream);
			MatchCollection mc = Regex.Matches(str, "\\* " + op + " \\(.*\\) \".*?\" \"(.*?)\"",
				RegexOptions.IgnoreCase | RegexOptions.Multiline);
			// u* LIST (\HasChildren) "." "INBOX"v ƂB
			foreach (Match m in mc) {
				if (!m.Success) continue; // C
				dirsTmp.Add(m.Groups[1].Value);
			}
		}

		/// <summary>
		/// JgfBNgݒB
		/// </summary>
		private void Examine(Stream stream, string dir) {
			if (dir != currentDir) {
				Utility.Send(stream, clientName + " EXAMINE " + dir + "\r\n");
				string str = Receive(stream);
				if (str.Contains(clientName + " OK")) {
					currentDir = dir;
				}
			}
		}

		/// <summary>
		/// 
		/// </summary>
		private List<string> InnerSearch(Stream stream, string type) {
			Utility.Send(stream, clientName + " UID SEARCH " + type + "\r\n");
			string str = Receive(stream);
			Match m = Regex.Match(str, "\\* SEARCH((\\s+\\d+)*)");
			if (m.Success) {
				List<string> uids = new List<string>();
				foreach (string s in m.Groups[1].Value.Split(new char[] { ' ', '\t' },
					StringSplitOptions.RemoveEmptyEntries)) {
					uids.Add(s);
				}
				return uids;
			}
			throw new IOException("[̎擾Ɏs܂");
		}

		/// <summary>
		/// M
		/// </summary>
		private string Receive(Stream stream) {
			Regex recvRegex = new Regex("^\\s*" + clientName + " (.+)$",
				RegexOptions.Compiled | RegexOptions.IgnoreCase | RegexOptions.Multiline);

			StringBuilder str = new StringBuilder();
			while (true) {
				str.Append(Utility.Recv(stream));
				Match m = recvRegex.Match(str.ToString());
				if (m.Success) {
					string lower = m.Groups[1].Value.ToLower();
					if (lower.Contains("ok ") || lower.Contains("ok.")) {
						return str.ToString(0, m.Index);
					}
					// G[ۂ
					throw new IOException("IMAPʐMɃG[܂: " + m.Groups[1]);
				}
			}
		}
	}
}
