using System;
using System.Collections.Generic;
using System.Text;

namespace Toolkit.Time {
	/// <summary>
	/// FNXB
	/// </summary>
	[Serializable]
	public class HourMinute : ICloneable, IComparable<HourMinute>, IEquatable<HourMinute> {
		int hour;
		int minute;

		public int Hour {
			get { return hour; }
			set {
				Validate(value, minute);
				hour = value;
			}
		}

		public int Minute {
			get { return minute; }
			set {
				Validate(hour, value);
				minute = value;
			}
		}

		public HourMinute() {
			hour = 0;
			minute = 0;
		}

		public HourMinute(int h, int m) {
			Validate(h, m);
			hour = h;
			minute = m;
		}

		static void Validate(int hour, int minute) {
			if (hour < 0 || 23 < hour) {
				throw new ArgumentOutOfRangeException("Hour");
			}
			if (minute < 0 || 59 < minute) {
				throw new ArgumentOutOfRangeException("Minute");
			}
		}

		#region Min, Max, Now

		/// <summary>
		/// ŏli00:00jԂ
		/// </summary>
		public static HourMinute Min {
			get { return new HourMinute(0, 0); }
		}

		/// <summary>
		/// őli23:59jԂ
		/// </summary>
		public static HourMinute Max {
			get { return new HourMinute(23, 59); }
		}

		/// <summary>
		/// ݎ̎擾
		/// </summary>
		public static HourMinute Now {
			get {
				HourMinute time = new HourMinute();
				DateTime now = DateTime.Now;
				time.Hour = now.Hour;
				time.Minute = now.Minute;
				return time;
			}
		}

		#endregion

		#region ToString(), TryParse(), Parse()

		/// <summary>
		/// 
		/// </summary>
		public override string ToString() {
			return Hour.ToString("d2") + ":" + Minute.ToString("d2");
		}

		/// <summary>
		/// 񂩂쐬B
		/// </summary>
		public static bool TryParse(string str, out HourMinute result) {
			string[] ss = str.Split(':');
			int h, m;
			if (ss.Length == 2 &&
				int.TryParse(ss[0], out h) &&
				int.TryParse(ss[1], out m)) {
				result = new HourMinute(h, m);
				return true;
			}
			result = new HourMinute();
			return false;
		}

		/// <summary>
		/// 񂩂쐬B
		/// sScheduleTime()Ԃ
		/// </summary>
		public static HourMinute Parse(string str) {
			HourMinute time;
			if (!TryParse(str, out time)) {
				System.Diagnostics.Debug.Fail("HourMinute.Parse()s");
			}
			return time;
		}

		#endregion

		#region IComparable<HourMinute> o

		public int CompareTo(HourMinute other) {
			// {int.ConpareTo()Ȃƃ_
			// ł̓I[o[t[̐Sz͂Ȃ̂ňZOKB
			if (hour != other.hour) {
				return hour - other.hour;
			}
			return minute - other.minute;
		}

		#endregion

		#region IEquatable<HourMinute> o

		public bool Equals(HourMinute other) {
			return CompareTo(other) == 0;
		}

		#endregion

		public override bool Equals(object obj) {
			HourMinute other = obj as HourMinute;
			return other != null && Equals(other);
		}

		public override int GetHashCode() {
			return hour.GetHashCode() ^ minute.GetHashCode();
		}

		/// <summary>
		/// ̍쐬
		/// </summary>
		public HourMinute Clone() {
			return (HourMinute)MemberwiseClone();
		}

		#region ICloneable o

		object ICloneable.Clone() {
			return Clone();
		}

		#endregion
	}
}
