using System;
using System.Collections.Generic;
using System.Text;

namespace Toolkit.Time {
	/// <summary>
	/// Ɠt̒lNX
	/// </summary>
	[Serializable]
	public class MonthDay : ICloneable, IComparable<MonthDay>, IEquatable<MonthDay> {
		int month;
		int day;

		public int Month {
			get { return month; }
			set {
				Validate(value, day);
				month = value;
			}
		}

		public int Day {
			get { return day; }
			set {
				Validate(month, value);
				day = value;
			}
		}

		public MonthDay() {
			month = 1;
			day = 1;
		}

		public MonthDay(int m, int d) {
			Validate(m, d);
			month = m;
			day = d;
		}

		static void Validate(int month, int day) {
			if (month < 1 || 12 < month) {
				throw new ArgumentOutOfRangeException("Month");
			}
			if (day < 1 || 31 < day) {
				throw new ArgumentOutOfRangeException("Day");
			}
		}

		#region Min, Max, Now

		/// <summary>
		/// ŏli01/01jԂ
		/// </summary>
		public static MonthDay Min {
			get { return new MonthDay(1, 1); }
		}

		/// <summary>
		/// őli12/31jԂ
		/// </summary>
		public static MonthDay Max {
			get { return new MonthDay(12, 31); }
		}

		/// <summary>
		/// ftHglṕAݎ̎擾
		/// </summary>
		public static MonthDay Now {
			get {
				MonthDay date = new MonthDay();
				DateTime now = DateTime.Now;
				date.Month = now.Month;
				date.Day = now.Day;
				return date;
			}
		}

		#endregion

		#region ToString(), TryParse(), Parse()

		/// <summary>
		/// 
		/// </summary>
		public override string ToString() {
			return Month.ToString("d2") + "/" + Day.ToString("d2");
		}

		/// <summary>
		/// 񂩂쐬B
		/// </summary>
		public static bool TryParse(string s, out MonthDay result) {
			string[] ss = s.Split('/');
			int h, m;
			if (ss.Length == 2 &&
				int.TryParse(ss[0], out h) &&
				int.TryParse(ss[1], out m)) {
				result = new MonthDay(h, m);
				return true;
			}
			result = new MonthDay();
			return false;
		}

		/// <summary>
		/// 񂩂쐬B
		/// sScheduleDate()Ԃ
		/// </summary>
		public static MonthDay Parse(string s) {
			MonthDay date;
			if (!TryParse(s, out date)) {
				System.Diagnostics.Debug.Fail("MonthDay.Parse()Ɏs");
			}
			return date;
		}

		#endregion

		#region IComparable<MonthDay> o

		public int CompareTo(MonthDay other) {
			// {int.ConpareTo()Ȃƃ_
			// ł̓I[o[t[̐Sz͂Ȃ̂ňZOKB
			if (month != other.month) {
				return month - other.month;
			}
			return day - other.day;
		}

		#endregion

		#region IEquatable<MonthDay> o

		public bool Equals(MonthDay other) {
			return CompareTo(other) == 0;
		}

		#endregion

		public override bool Equals(object obj) {
			MonthDay other = obj as MonthDay;
			return other != null && Equals(other);
		}

		public override int GetHashCode() {
			return month.GetHashCode() ^ day.GetHashCode();
		}

		/// <summary>
		/// ̍쐬
		/// </summary>
		public MonthDay Clone() {
			return (MonthDay)MemberwiseClone();
		}

		#region ICloneable o

		object ICloneable.Clone() {
			return Clone();
		}

		#endregion
	}
}
