using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Toolkit.Updater;
using Info = Toolkit.Updater.Info;
using System.IO;

namespace UpdateInfoEditor {
	public partial class MainForm : Form {
		string fileName;
		Info.UpdateInfo info;

		/// <summary>
		/// B
		/// </summary>
		/// <param name="fileName">ݒt@C</param>
		public MainForm(string fileName) {
			InitializeComponent();

			this.fileName = Path.GetFullPath(fileName);
			try {
				info = Info.UpdateInfo.DeserializeFromFile<Info.UpdateInfo>(fileName);
			} catch (System.IO.FileNotFoundException) {
				info = new Info.UpdateInfo();
			}

			Toolkit.Forms.Utility.DisableCloseButton(this);

			Toolkit.Forms.Utility.SetArray(listBox1, info.Files);
			Toolkit.Forms.Utility.SetArray(listBox2, info.Entries);
			Toolkit.Forms.Utility.SetArray(listBox3, info.Notices);

#if DEBUG
			button16.Visible = true;
#endif
		}

		private void buttonOk_Click(object sender, EventArgs e) {
			Close();
		}

		/// <summary>
		/// nB
		/// </summary>
		private void MainForm_FormClosing(object sender, FormClosingEventArgs e) {
			info.Files = Toolkit.Forms.Utility.GetArray<Info.File>(listBox1);
			info.Entries = Toolkit.Forms.Utility.GetArray<Info.Entry>(listBox2);
			info.Notices = Toolkit.Forms.Utility.GetArray<Info.Notice>(listBox3);
			// hashƂݒ
			string baseDir = Path.GetDirectoryName(fileName);
			foreach (Info.File file in info.Files) {
				string fn = file.RelativeUrl.Replace('/', Path.DirectorySeparatorChar);
				fn = Path.Combine(baseDir, fn);
				file.FileSize = (int)new FileInfo(fn).Length;
				file.Hash = Toolkit.Hash.MD5(File.ReadAllBytes(fn));
			}

			info.SerializeToFile(fileName);
		}

		/// <summary>
		/// ǉ
		/// </summary>
		private void button1_Click(object sender, EventArgs e) {
			using (FileConfigForm form = new FileConfigForm(
				new Info.File(), Path.GetDirectoryName(fileName))) {
				if (form.ShowDialog(this) == DialogResult.OK) {
					listBox1.Items.Add(form.Config);
				}
			}
		}

		/// <summary>
		/// ҏW
		/// </summary>
		private void button2_Click(object sender, EventArgs e) {
			if (listBox1.SelectedIndex < 0) return;
			using (FileConfigForm form = new FileConfigForm(
				(Info.File)listBox1.SelectedItem, Path.GetDirectoryName(fileName))) {
				if (form.ShowDialog(this) == DialogResult.OK) {
					listBox1.Items[listBox1.SelectedIndex] = form.Config;
				}
			}
		}

		/// <summary>
		/// 폜
		/// </summary>
		private void button3_Click(object sender, EventArgs e) {
			if (listBox1.SelectedIndex < 0) return;
			if (MessageBox.Show(this, listBox1.SelectedItem + " 폜܂", "mF",
				MessageBoxButtons.OKCancel) == DialogResult.OK) {
				Toolkit.Forms.Utility.RemoveSelected(listBox1);
			}
		}

		/// <summary>
		/// ǉ
		/// </summary>
		private void button6_Click(object sender, EventArgs e) {
			using (EntryConfigForm form = new EntryConfigForm(new Info.Entry())) {
				if (form.ShowDialog(this) == DialogResult.OK) {
					listBox2.Items.Add(form.Config);
				}
			}
		}

		/// <summary>
		/// ҏW
		/// </summary>
		private void button5_Click(object sender, EventArgs e) {
			if (listBox2.SelectedIndex < 0) return;
			using (EntryConfigForm form = new EntryConfigForm((Info.Entry)listBox2.SelectedItem)) {
				if (form.ShowDialog(this) == DialogResult.OK) {
					listBox2.Items[listBox2.SelectedIndex] = form.Config;
				}
			}
		}

		/// <summary>
		/// 폜
		/// </summary>
		private void button4_Click(object sender, EventArgs e) {
			if (listBox2.SelectedIndex < 0) return;
			if (MessageBox.Show(this, listBox2.SelectedItem + " 폜܂", "mF",
				MessageBoxButtons.OKCancel) == DialogResult.OK) {
				Toolkit.Forms.Utility.RemoveSelected(listBox2);
			}
		}

		/// <summary>
		/// ǉ
		/// </summary>
		private void button9_Click(object sender, EventArgs e) {
			using (NoticeConfigForm form = new NoticeConfigForm(new Info.Notice())) {
				if (form.ShowDialog(this) == DialogResult.OK) {
					listBox3.Items.Add(form.Config);
				}
			}
		}

		/// <summary>
		/// ҏW
		/// </summary>
		private void button8_Click(object sender, EventArgs e) {
			if (listBox3.SelectedIndex < 0) return;
			using (NoticeConfigForm form = new NoticeConfigForm((Info.Notice)listBox3.SelectedItem)) {
				if (form.ShowDialog(this) == DialogResult.OK) {
					listBox3.Items[listBox3.SelectedIndex] = form.Config;
				}
			}
		}

		/// <summary>
		/// 폜
		/// </summary>
		private void button7_Click(object sender, EventArgs e) {
			if (listBox3.SelectedIndex < 0) return;
			if (MessageBox.Show(this, listBox3.SelectedItem + " 폜܂", "mF",
				MessageBoxButtons.OKCancel) == DialogResult.OK) {
				Toolkit.Forms.Utility.RemoveSelected(listBox3);
			}
		}

		private void listBox1_DoubleClick(object sender, EventArgs e) {
			button2_Click(this, null);
		}

		private void listBox2_DoubleClick(object sender, EventArgs e) {
			button5_Click(this, null);
		}

		private void listBox3_DoubleClick(object sender, EventArgs e) {
			button8_Click(this, null);
		}

		#region 

		/// <summary>
		/// 
		/// </summary>
		private void button10_Click(object sender, EventArgs e) {
			Toolkit.Forms.Utility.UpSelected(listBox1);
		}

		/// <summary>
		/// 
		/// </summary>
		private void button11_Click(object sender, EventArgs e) {
			Toolkit.Forms.Utility.DownSelected(listBox1);
		}

		/// <summary>
		/// 
		/// </summary>
		private void button12_Click(object sender, EventArgs e) {
			Toolkit.Forms.Utility.UpSelected(listBox2);
		}

		/// <summary>
		/// 
		/// </summary>
		private void button13_Click(object sender, EventArgs e) {
			Toolkit.Forms.Utility.DownSelected(listBox2);
		}

		/// <summary>
		/// 
		/// </summary>
		private void button14_Click(object sender, EventArgs e) {
			Toolkit.Forms.Utility.UpSelected(listBox3);
		}

		/// <summary>
		/// 
		/// </summary>
		private void button15_Click(object sender, EventArgs e) {
			Toolkit.Forms.Utility.DownSelected(listBox3);
		}

		#endregion

		/// <summary>
		/// ErrorReportereXg{^
		/// </summary>
		private void button16_Click(object sender, EventArgs e) {
			throw new ApplicationException("eXg",
				new IOException("ErrorReportereXg"));
		}
	}
}