﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.IO;

namespace html2cfg
{
	class Program
	{
		[STAThread]
		static void Main(string[] args) {
			string fileName;
			if (args.Length == 1 && File.Exists(args[0])) {
				fileName = args[0];
			} else {
				using (OpenFileDialog dialog = new OpenFileDialog()) {
					dialog.AddExtension = true;
					dialog.CheckFileExists = true;
					dialog.CheckPathExists = true;
					dialog.DereferenceLinks = true;
					dialog.Multiselect = false;
					dialog.RestoreDirectory = true;
					dialog.FileName = "パスワード.html";
					dialog.Filter = "HTMLファイル(*.html)|*.html|全てのファイル(*.*)|*.*";
					dialog.Title = "パスワードファイル(html)の選択";
					if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK) {
						fileName = dialog.FileName;
					} else {
						// おしまい。
						return;
					}
				}
			}

			string htmlData = File.ReadAllText(fileName, Encoding.Default);

			using (ぱすわど.ConfigManager configManager = new ぱすわど.ConfigManager()) {
				System.Diagnostics.Debug.Assert(configManager.IsReady);

				Regex regex = new Regex(@"^\s+<tr><td>(.*)</td><td>(.*)</td>" +
					@"<td>(.*)</td><td>(.*)</td><td>(.*)</td></tr>\s*$", RegexOptions.Compiled | RegexOptions.Multiline);
				foreach (Match match in regex.Matches(htmlData)) {
					if (match.Groups.Count == 1 + 5) {
						ぱすわど.Item item = new ぱすわど.Item();
						item.Name = match.Groups[1].Value;
						item.User = match.Groups[2].Value;
						item.Pass = match.Groups[3].Value;
						item.Mail = match.Groups[4].Value;
						item.Memo = match.Groups[5].Value.Replace("<br>", "\n");
						configManager.ItemManager.Root.Items.Add(item);
					} else {
						System.Diagnostics.Debug.Fail("グループ数が合わないエラー");
					}
				}
			}

			System.Diagnostics.Debug.Assert(File.Exists(Toolkit.Serializable.DefaultBaseName + ".cfg"));

			string saveDir = Path.GetDirectoryName(System.Diagnostics.Process.GetCurrentProcess().MainModule.FileName);
			string saveFileName = Path.Combine(saveDir, "ぱすわど.cfg");
			if (File.Exists(saveFileName)) {
				using (SaveFileDialog dialog = new SaveFileDialog()) {
					dialog.AddExtension = true;
					dialog.CheckFileExists = false;
					dialog.CheckPathExists = true;
					dialog.DereferenceLinks = true;
					dialog.RestoreDirectory = true;
					dialog.OverwritePrompt = true;
					dialog.InitialDirectory = saveDir;
					dialog.FileName = "ぱすわど.cfg";
					dialog.Filter = "CFGファイル(*.cfg)|*.cfg|全てのファイル(*.*)|*.*";
					dialog.Title = "コンフィグファイル保存先の選択";
					if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK) {
						string srcName = Toolkit.Serializable.DefaultBaseName + ".cfg";
						File.Copy(srcName, dialog.FileName, true);
						File.Delete(srcName);
					}
				}
			} else {
				File.Move(Toolkit.Serializable.DefaultBaseName + ".cfg", saveFileName);
			}
		}
	}
}
