﻿using System;
using System.Collections.Generic;
using System.Text;

namespace ぱすわど {
	public class ConfigManager : IDisposable {
		Config config = new Config();
		ItemManager itemManager = null;
		string masterPass = null;
		int suspendSave = 0;

		public Config Config { get { return config; } }
		public ItemManager ItemManager { get { return itemManager; } }
		public string MasterPass {
			get { return masterPass; }
			set { masterPass = value; }
		}

		/// <summary>
		/// 一時的にセーブしなくする。
		/// </summary>
		public void SuspendSave() {
			lock (config) {
				suspendSave++;
			}
		}
		/// <summary>
		/// Suspend解除
		/// </summary>
		public void ResumeSave() {
			lock (config) {
				suspendSave--;
			}
		}

		public class SaveSuspender : IDisposable {
			ConfigManager owner;

			public SaveSuspender(ConfigManager owner) {
				this.owner = owner;
				owner.SuspendSave();
			}

			public void Dispose() {
				owner.ResumeSave();
			}
		}

		/// <summary>
		/// 暗号化解除が終わったり、デフォルトのが生成済みならtrue
		/// </summary>
		public bool IsReady { get { return itemManager != null; } }

		public ConfigManager() {
			config = Config.Deserialize();
			// データが空ならデフォルトを作成
			if (config.Data == null || config.Data.Length <= 0) {
				itemManager = new ItemManager();
				masterPass = "";
			} else {
				// 空パスワードなら入力させるまでもなく解除しとく。
				Decrypt("");
			}
		}

		public void Dispose() {
			// ここでの保存は念のため。本来は変更直後にすぐ保存するべし。
			Save();
		}

		/// <summary>
		/// 暗号解除処理
		/// </summary>
		/// <returns>成功したらtrue、失敗したらfalse</returns>
		public bool Decrypt(string masterPass) {
			try {
				Toolkit.AESCrypt crypt = new Toolkit.AESCrypt(masterPass);
				string data = crypt.DecodeString(config.Data);
				if (config.Hash != Toolkit.Hash.MD5(data)) {
					throw new Exception("ハッシュ未一致"); // エラー
				}
				itemManager = new ItemManager(data);
				this.masterPass = masterPass;
				return true;
			} catch {
				itemManager = null;
				masterPass = null;
				return false;
			}
		}

		/// <summary>
		/// 暗号化して保存
		/// </summary>
		public void Save() {
			if (0 < suspendSave) return;
			if (IsReady) {
				Toolkit.AESCrypt crypt = new Toolkit.AESCrypt(masterPass);
				string data = itemManager.String;
				config.Data = crypt.EncodeString(data);
				config.Hash = Toolkit.Hash.MD5(data);
				config.Serialize();
			}
		}
	}
}
