﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

namespace ぱすわど {
	/// <summary>
	/// データの保存と読み込みとかを行うオブジェクト。
	/// </summary>
	public class ItemManager {
		ItemDirectory root;

		public ItemManager() {
			root = new ItemDirectory("root");
		}
		public ItemManager(string str) {
			String = str;
		}

		/// <summary>
		/// データの読み込みと書き込みと。
		/// </summary>
		public string String {
			get {
				return root.SerializeToString();
			}
			set {
				// 空ならrootだけ作っておく。
				if (value == null || value.Length <= 0) {
					root = new ItemDirectory("root");
					return;
				}
				root = Toolkit.Serializable.DeserializeFromString<ItemDirectory>(value);
			}
		}

		/// <summary>
		/// 根っこを取得
		/// </summary>
		public ItemDirectory Root {
			get { return root; }
		}

		/// <summary>
		/// 名前に使えない文字を含んでいたらfalse。
		/// なんかの時に困ることがあるような気がしなくもないので、ファイルシステム的にダメな文字はダメって事にしといてみる。
		/// </summary>
		public static bool IsValidName(string name) {
			bool invalid =
				name.Contains("\\") || name.Contains("/") || name.Contains(":") ||
				name.Contains("*") || name.Contains("\"") || name.Contains("?") ||
				name.Contains("<") || name.Contains(">") || name.Contains("|");
			return !invalid;
		}
	}

	/// <summary>
	/// でぃれくとり。
	/// </summary>
	[Serializable]
	public class ItemDirectory : Toolkit.Serializable, ICloneable {
		List<ItemDirectory> directories = new List<ItemDirectory>();
		List<Item> items = new List<Item>();
		string name; // ディレクトリ名

		public ItemDirectory() {
		}
		public ItemDirectory(string str) {
			name = str;
		}

		/// <summary>
		/// 複製の作成
		/// </summary>
		public ItemDirectory Clone() {
			ItemDirectory copy = (ItemDirectory)MemberwiseClone();
			// string以外の参照型なメンバがあればここでコピー
			copy.directories = Toolkit.Utility.Clone(directories);
			copy.items = Toolkit.Utility.Clone(items);
			return copy;
		}

		#region ICloneable メンバ

		object ICloneable.Clone() {
			return Clone();
		}

		#endregion

		public string Name {
			get { return name; }
			set { name = value; }
		}
		public List<ItemDirectory> Directories {
			get { return directories; }
		}
		public List<Item> Items {
			get { return items; }
		}

		public override string ToString() {
			return "ディレクトリ：" + name;
		}

		/// <summary>
		/// 親ディレクトリを探し出す
		/// </summary>
		public ItemDirectory FindParent(ItemDirectory item) {
			if (directories.Contains(item)) return this;
			foreach (ItemDirectory child in directories) {
				ItemDirectory parent = child.FindParent(item);
				if (parent != null) return parent;
			}
			return null;
		}
		/// <summary>
		/// 親ディレクトリを探し出す
		/// </summary>
		public ItemDirectory FindParent(Item item) {
			if (items.Contains(item)) return this;
			foreach (ItemDirectory child in directories) {
				ItemDirectory parent = child.FindParent(item);
				if (parent != null) return parent;
			}
			return null;
		}
	}

	/// <summary>
	/// パスワードとかのデータ1件。
	/// </summary>
	[Serializable]
	public class Item : ICloneable {
		public string Name = "";
		public string User = "";
		public string Pass = "";
		public string Mail = "";
		public string Url = "";
		public string Memo = "";
		public string Custom1Name = "";
		public string Custom1Data = "";
		public bool Custom1Hide = false;
		public string Custom2Name = "";
		public string Custom2Data = "";
		public bool Custom2Hide = false;
		public string Custom3Name = "";
		public string Custom3Data = "";
		public bool Custom3Hide = false;
		public string Custom4Name = "";
		public string Custom4Data = "";
		public bool Custom4Hide = false;

		public Item() {
		}
		public Item(string str) {
			Name = str;
		}

		/// <summary>
		/// 複製の作成
		/// </summary>
		public Item Clone() {
			Item copy = (Item)MemberwiseClone();
			// string以外の参照型なメンバがあればここでコピー
			return copy;
		}

		#region ICloneable メンバ

		object ICloneable.Clone() {
			return Clone();
		}

		#endregion

		public override string ToString() {
			return "データ：" + Name;
		}
	}
}