﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ぱすわど {
	public partial class MainForm : Form {
		ConfigManager configManager;
		Data data = new Data();
		ItemDirectory currentDir = null;
		Item currentItem = null;
		object lockObject = new object();

		public MainForm(ConfigManager configManager) {
			InitializeComponent();

			this.configManager = configManager;

			data = Data.Deserialize();

			int build = System.Diagnostics.Process.GetCurrentProcess()
				.MainModule.FileVersionInfo.FileBuildPart;
			Text = "ぱすわど build-" + build;

			Size = data.WindowSize;
			Toolkit.Forms.Utility.SetLocationWithClip(this, data.WindowPos);
			splitContainer1.SplitterDistance = data.TreeWidth;
			for (int i = 0, n = data.ColmunWidths.Length; i < n; i++) {
				listView1.Columns[i].Width = data.ColmunWidths[i];
			}
			// ツリービューの生成
			TreeNode root = CreateTreeNode(configManager.ItemManager.Root);
			root.ImageIndex = -1;
			treeView1.Nodes.Add(root);
			treeView1.ExpandAll();
			// ツリービューとかの初期状態をセット
			SetCurrentDir(configManager.ItemManager.Root);
		}

		/// <summary>
		/// 後始末。
		/// </summary>
		private void MainForm_FormClosing(object sender, FormClosingEventArgs e) {
			data.WindowPos = Location;
			data.WindowSize = Size;
			data.TreeWidth = splitContainer1.SplitterDistance;
			for (int i = 0, n = data.ColmunWidths.Length; i < n; i++) {
				data.ColmunWidths[i] = listView1.Columns[i].Width;
			}
			data.Serialize();
		}

		#region ツリービューやリストビューのヘルパっぽいのとか。

		/// <summary>
		/// カレントディレクトリの設定とツリービューの更新とか。
		/// </summary>
		private void SetCurrentDir(ItemDirectory dir) {
			if (currentDir != dir) {
				currentDir = dir;
				// アイテムは未選択状態にしとく。
				SetCurrentItem(null);
				// リストビューの更新
				listView1.Items.Clear();
				foreach (ItemDirectory item in currentDir.Directories) {
					listView1.Items.Add(CreateListViewItem(item));
				}
				foreach (Item item in currentDir.Items) {
					listView1.Items.Add(CreateListViewItem(item));
				}
				// ツリービュの更新
				treeView1.SelectedNode = FindTreeNode(currentDir);
			}
		}

		/// <summary>
		/// 子ノードを再帰的に生成
		/// </summary>
		private TreeNode[] GetChildren(ItemDirectory dir) {
			TreeNode[] children = new TreeNode[dir.Directories.Count];
			int i = 0;
			foreach (ItemDirectory child in dir.Directories) {
				children[i++] = CreateTreeNode(child);
			}
			return children;
		}

		/// <summary>
		/// 新しいTreeNodeオブジェクトを作成
		/// </summary>
		private TreeNode CreateTreeNode(ItemDirectory item) {
			TreeNode node = new TreeNode(item.Name);
			node.Nodes.AddRange(GetChildren(item)); // 再帰的に孫を追加
			node.ExpandAll(); // ツリーは展開しておく
			node.Tag = item; // ←好きなもの入れていいらしい
			node.ImageIndex = 0; // 0番のアイコンを使用
			return node;
		}

		/// <summary>
		/// ItemDirectoryオブジェクトからTreeNodeを探し出す
		/// </summary>
		private TreeNode FindTreeNode(ItemDirectory dir) {
			return Toolkit.Forms.Utility.Find(treeView1,
				delegate(TreeNode n) { return n.Tag == dir; });
		}

		/// <summary>
		/// リストビューでアイテムが選択されたりしたときに呼ばれる
		/// </summary>
		private void SetCurrentItem(ListViewItem i) {
			lock (lockObject) {
				Item item = i == null ? null : i.Tag as Item;
				panel1.SuspendLayout();
				try {
					currentItem = null;
					using (ConfigManager.SaveSuspender s = new ConfigManager.SaveSuspender(configManager)) {
						// ↑意味無いかも
						InnerSetCurrentItem(item);
					}
				} finally {
					panel1.ResumeLayout();
					currentItem = item;
				}
			}
		}

		private void InnerSetCurrentItem(Item item) {
			bool enable = item != null;
			SetEnable(textBox1, enable);
			SetEnable(textBox2, enable);
			SetEnable(textBox3, enable);
			SetEnable(textBox5, enable);
			SetEnable(textBox4, enable);
			SetEnable(button1, enable);
			SetEnable(button2, enable);
			SetEnable(button3, enable);
			SetEnable(button5, enable);
			SetEnable(button4, enable);
			SetEnable(button6, enable);
			SetEnable(button7, enable);
			SetEnable(button8, enable);
			SetEnable(button9, enable);
			SetEnable(textBox6, enable);
			SetEnable(textBox7, enable);
			SetEnable(textBox9, enable);
			SetEnable(textBox8, enable);
			SetEnable(textBox10, enable);
			SetEnable(textBox11, enable);
			SetEnable(textBox13, enable);
			SetEnable(textBox12, enable);
			SetEnable(button12, enable);
			SetEnable(button11, enable);
			SetEnable(button10, enable);
			SetEnable(checkBox1, enable);
			SetEnable(button15, enable);
			SetEnable(button14, enable);
			SetEnable(button13, enable);
			SetEnable(checkBox2, enable);
			SetEnable(button18, enable);
			SetEnable(button17, enable);
			SetEnable(button16, enable);
			SetEnable(checkBox3, enable);
			SetEnable(button21, enable);
			SetEnable(button20, enable);
			SetEnable(button19, enable);
			SetEnable(checkBox4, enable);
			if (enable) {
				textBox1.Text = item.User;
				textBox2.Text = item.Pass;
				textBox3.Text = item.Mail;
				textBox5.Text = item.Url;
				textBox4.Text = item.Memo.Replace("\n", "\r\n").Replace("\r\r\n", "\r\n"); // どうもここ、set/getが非対称ぽい。。
				textBox6.Text = item.Custom1Name;
				textBox7.Text = item.Custom1Data;
				checkBox1.Checked = item.Custom1Hide;
				textBox9.Text = item.Custom2Name;
				textBox8.Text = item.Custom2Data;
				checkBox2.Checked = item.Custom2Hide;
				textBox11.Text = item.Custom3Name;
				textBox10.Text = item.Custom3Data;
				checkBox3.Checked = item.Custom3Hide;
				textBox13.Text = item.Custom4Name;
				textBox12.Text = item.Custom4Data;
				checkBox4.Checked = item.Custom4Hide;
				// ↓TextBoxとは違ってこれ呼ばれないっぽいので。。
				checkBox1_CheckedChanged(this, EventArgs.Empty);
				checkBox2_CheckedChanged(this, EventArgs.Empty);
				checkBox3_CheckedChanged(this, EventArgs.Empty);
				checkBox4_CheckedChanged(this, EventArgs.Empty);
			} else { // 一応無効なときは空にする
				textBox1.Text = "";
				textBox2.Text = "";
				textBox3.Text = "";
				textBox5.Text = "";
				textBox4.Text = "";
				textBox6.Text = "";
				textBox7.Text = "";
				textBox9.Text = "";
				textBox8.Text = "";
				textBox10.Text = "";
				textBox11.Text = "";
				textBox13.Text = "";
				textBox12.Text = "";
				checkBox1.Checked = false;
				checkBox2.Checked = false;
				checkBox3.Checked = false;
				checkBox4.Checked = false;
			}
		}

		void SetEnable(Control c, bool enable) {
			if (c.Enabled != enable) {
				c.Enabled = enable;
			}
		}

		/// <summary>
		/// ListViewItemの作成
		/// </summary>
		private ListViewItem CreateListViewItem(ItemDirectory data) {
			ListViewItem lvItem = new ListViewItem(data.Name);
			lvItem.Tag = data;
			lvItem.ImageIndex = 0;
			return lvItem;
		}
		/// <summary>
		/// ListViewItemの作成
		/// </summary>
		private ListViewItem CreateListViewItem(Item data) {
			ListViewItem lvItem = new ListViewItem(data.Name);
			lvItem.Tag = data;
			lvItem.ImageIndex = 1;
			lvItem.SubItems.Add(data.User);
			lvItem.SubItems.Add(data.Mail);
			lvItem.SubItems.Add(data.Url);
			lvItem.SubItems.Add(data.Memo);
			return lvItem;
		}

		/// <summary>
		/// ディレクトリの追加
		/// </summary>
		private void Add(ItemDirectory dir, ItemDirectory newData, bool beginEdit) {
			// リストビューの更新
			if (dir == currentDir) {
				ListViewItem lvItem = CreateListViewItem(newData);
				listView1.Items.Insert(dir.Directories.Count, lvItem);
				lvItem.Selected = true;
				if (beginEdit) {
					lvItem.BeginEdit();
				}
			}
			// ツリービューの更新
			TreeNode node = FindTreeNode(dir);
			if (node != null) {
				node.Nodes.Add(CreateTreeNode(newData));
				node.ExpandAll();
			}
			// dirに追加
			dir.Directories.Add(newData);
		}

		/// <summary>
		/// アイテムの追加
		/// </summary>
		private void Add(ItemDirectory dir, Item newData, bool beginEdit) {
			// リストビューの更新
			if (dir == currentDir) {
				ListViewItem lvItem = CreateListViewItem(newData);
				listView1.Items.Add(lvItem);
				lvItem.Selected = true;
				if (beginEdit) {
					lvItem.BeginEdit();
				}
			}
			// dirに追加
			dir.Items.Add(newData);
		}
		/// <summary>
		/// アイテムの挿入。indexはリストビュー上でのindex
		/// </summary>
		private void Insert(ItemDirectory dir, Item newItem, int index) {
			// リストビューの更新
			if (dir == currentDir) {
				ListViewItem newLvItem = CreateListViewItem(newItem);
				listView1.Items.Insert(index, newLvItem);
				newLvItem.Selected = true;
			}
			// dirに追加
			index -= dir.Directories.Count; // ディレクトリが先頭にあるので差し引く
			dir.Items.Insert(index, newItem);
		}

		/// <summary>
		/// ディレクトリの削除
		/// </summary>
		private void Remove(ItemDirectory dir, ItemDirectory remove) {
			Toolkit.Forms.Utility.RemoveAll(listView1, delegate(ListViewItem n) { return n.Tag == remove; });
			Toolkit.Forms.Utility.RemoveAll(treeView1, delegate(TreeNode n) { return n.Tag == remove; });
			bool b = dir.Directories.Remove(remove);
			System.Diagnostics.Debug.Assert(b);
			// 削除したとこがカレントディレクトリなら、その親へ移動。
			if (currentDir == remove) {
				SetCurrentDir(dir);
			}
		}
		/// <summary>
		/// アイテムの削除
		/// </summary>
		private void Remove(ItemDirectory dir, Item item) {
			Toolkit.Forms.Utility.RemoveAll(listView1, delegate(ListViewItem n) { return n.Tag == item; });
			bool b = dir.Items.Remove(item);
			System.Diagnostics.Debug.Assert(b);
		}

		#endregion

		#region 右クリックメニューな辺り

		private void フォルダを追加DToolStripMenuItem_Click(object sender, EventArgs e) {
			Add(currentDir, new ItemDirectory("新しいフォルダ"), true);
			configManager.Save();
		}

		private void アイテムを追加FToolStripMenuItem_Click(object sender, EventArgs e) {
			Add(currentDir, new Item("新しいアイテム"), true);
			configManager.Save();
		}

		private void 名前を変更RToolStripMenuItem_Click(object sender, EventArgs e) {
			foreach (ListViewItem item in listView1.SelectedItems) {
				item.BeginEdit();
				break; // 最初の一つだけやってお終い。
			}
		}

		private void 削除EToolStripMenuItem_Click(object sender, EventArgs e) {
			foreach (ListViewItem selectedItem in listView1.SelectedItems) {
				ItemDirectory dir = selectedItem.Tag as ItemDirectory;
				if (dir != null) {
					if (MessageBox.Show(this, dir + " を削除しますか?\r\n\r\n※ 元には戻せないので注意してください。", "確認",
						MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes) {
						Remove(currentDir, dir);
						configManager.Save();
					}
					continue;
				}
				Item item = selectedItem.Tag as Item;
				if (item != null) {
					if (MessageBox.Show(this, item + " を削除しますか?\r\n\r\n※ 元には戻せないので注意してください。", "確認",
						MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes) {
						Remove(currentDir, item);
						configManager.Save();
					}
					continue;
				}
				// エラーなのだが。。
				System.Diagnostics.Debug.Fail("謎エラー");
			}
		}

		#endregion

		#region メニューバーな辺り

		private void 終了XToolStripMenuItem_Click(object sender, EventArgs e) {
			Close();
		}

		private void マスターパスワードの設定MToolStripMenuItem_Click(object sender, EventArgs e) {
			using (SetPassForm form = new SetPassForm(configManager)) {
				form.ShowDialog(this);
			}
		}

		private void 実行ファイルのあるフォルダを開くMToolStripMenuItem_Click(object sender, EventArgs e) {
			System.Diagnostics.Process.Start(System.IO.Path.GetDirectoryName(Application.ExecutablePath));
		}

		private void 配布サイトを開くWToolStripMenuItem_Click(object sender, EventArgs e) {
			System.Diagnostics.Process.Start("http://hp.vector.co.jp/authors/VA039571/");
		}

		private void 作者へメールを送るEToolStripMenuItem_Click(object sender, EventArgs e) {
			System.Diagnostics.Process.Start("http://tqzh.tk:24497/site/mail/");
		}

		private void ネットワーク更新設定OToolStripMenuItem_Click(object sender, EventArgs e) {
			Config config = configManager.Config;
			using (Toolkit.Updater.UpdateConfigForm form = new Toolkit.Updater.UpdateConfigForm(config.UpdateConfig)) {
				if (form.ShowDialog(this) == DialogResult.OK) {
					config.UpdateConfig = form.Config;
					config.Serialize();
				}
			}
		}

		private void ネットワーク更新NToolStripMenuItem_Click(object sender, EventArgs e) {
			Config config = configManager.Config;
			//string url = "http://tqzh.tk:24497/data/pw.cfg";
			string url = "http://hp.vector.co.jp/authors/VA039571/pw.cfg";
			using (Toolkit.Updater.UpdateForm form = new Toolkit.Updater.UpdateForm(url,
				config.UpdateConfig, data.UpdateRecord)) {
				form.ShowDialog(this);
				data.UpdateRecord = form.UpdateRecord;
				data.Serialize();
				if (form.Finished) {
					Close();
				}
			}
		}

		#endregion

		/// <summary>
		/// ツリービューのノードが選択された(カレントディレクトリの変更を行う)
		/// </summary>
		private void treeView1_AfterSelect(object sender, TreeViewEventArgs e) {
			ItemDirectory dir = e.Node.Tag as ItemDirectory;
			if (dir != null) {
				SetCurrentDir(dir);
			}
		}
		/// <summary>
		/// リストビューがダブルクリックされた(カレントディレクトリの変更を行う)
		/// </summary>
		private void listView1_DoubleClick(object sender, EventArgs e) {
			foreach (ListViewItem item in listView1.SelectedItems) {
				ItemDirectory dir = item.Tag as ItemDirectory;
				if (dir != null) {
					SetCurrentDir(dir);
					break;
				}
			}
		}
		/// <summary>
		/// アイテムが選択されたり選択が解除されたり
		/// </summary>
		private void listView1_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e) {
			SetCurrentItem(listView1.SelectedItems.Count == 1 ? listView1.SelectedItems[0] : null);
		}

		/// <summary>
		/// ラベルの編集終わったときの処理。
		/// </summary>
		private void listView1_AfterLabelEdit(object sender, LabelEditEventArgs e) {
			if (e.Label == null) return; // よく分からんけどなぜか呼ばれてくるので無視っとく。
			if (!ItemManager.IsValidName(e.Label)) {
				// 名前に使えない文字エラー
				MessageBox.Show(this, "名前に \\, /, :, *, \", ?, <, >, | は使用できません。", "エラー", MessageBoxButtons.OK);
				e.CancelEdit = true;
			} else {
				ListViewItem lvItem = listView1.Items[e.Item];
				ItemDirectory dir = lvItem.Tag as ItemDirectory;
				Item item = lvItem.Tag as Item;
				if (dir != null) {
					// ディレクトリな場合
					dir.Name = e.Label;
					TreeNode node = FindTreeNode(dir);
					if (node != null) {
						node.Text = e.Label;
					}
				} else if (item != null) {
					// えんとりな場合
					item.Name = e.Label;
				} else {
					// 謎エラー
					e.CancelEdit = true;
				}
				configManager.Save();
			}
		}
		/// <summary>
		/// ラベルの編集が終わったときの処理その２。
		/// </summary>
		private void treeView1_AfterLabelEdit(object sender, NodeLabelEditEventArgs e) {
			if (e.Label == null) return; // よく分からんけどなぜか呼ばれてくるので無視っとく。
			if (!ItemManager.IsValidName(e.Label)) {
				// 名前に使えない文字エラー
				MessageBox.Show(this, "名前に \\, /, :, *, \", ?, <, >, | は使用できません。", "エラー", MessageBoxButtons.OK);
				e.CancelEdit = true;
			} else {
				ItemDirectory dir = e.Node.Tag as ItemDirectory;
				if (dir != null) {
					// 名前変更ー
					dir.Name = e.Label;
				} else {
					// 謎エラー
					e.CancelEdit = true;
				}
				configManager.Save();
			}
		}

		/// <summary>
		/// リストビューのキー操作
		/// </summary>
		private void listView1_KeyUp(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.F2) { // 名前変更
				名前を変更RToolStripMenuItem_Click(this, null);
			} else if (e.KeyCode == Keys.Delete) { // 削除
				削除EToolStripMenuItem_Click(this, null);
			} else if (e.KeyCode == Keys.Enter) { // ダブルクリック扱い(ディレクトリなら移動)
				listView1_DoubleClick(this, null);
			}
		}
		/// <summary>
		/// ツリービューのキー操作
		/// </summary>
		private void treeView1_KeyUp(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.F2) { // 名前変更
				if (treeView1.SelectedNode != null) {
					treeView1.SelectedNode.BeginEdit();
				}
			} else if (e.KeyCode == Keys.Delete) { // 削除
				ItemDirectory parent = (ItemDirectory)treeView1.SelectedNode.Parent.Tag;
				ItemDirectory dir = (ItemDirectory)treeView1.SelectedNode.Tag;
				if (MessageBox.Show(this, dir + " を削除しますか?\r\n\r\n※ 元には戻せないので注意してください。", "確認",
					MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes) {
					Remove(parent, dir);
					configManager.Save();
				}
			}
		}

		/// <summary>
		/// ノード閉じる時呼ばれる処理。何が何でも閉じない。
		/// </summary>
		private void treeView1_BeforeCollapse(object sender, TreeViewCancelEventArgs e) {
			e.Cancel = true;
		}

		#region アイテム編集欄

		#region TextChanged

		private void textBox1_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.User = textBox1.Text;
				foreach (ListViewItem lvItem in listView1.SelectedItems) {
					if (lvItem.Tag == currentItem) {
						lvItem.SubItems[1].Text = currentItem.User;
					}
				}
				configManager.Save();
			}
		}

		private void textBox2_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Pass = textBox2.Text;
				configManager.Save();
			}
		}

		private void textBox3_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Mail = textBox3.Text;
				foreach (ListViewItem lvItem in listView1.SelectedItems) {
					if (lvItem.Tag == currentItem) {
						lvItem.SubItems[2].Text = currentItem.Mail;
					}
				}
				configManager.Save();
			}
		}

		private void textBox5_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Url = textBox5.Text;
				foreach (ListViewItem lvItem in listView1.SelectedItems) {
					if (lvItem.Tag == currentItem) {
						lvItem.SubItems[3].Text = currentItem.Url;
					}
				}
				configManager.Save();
			}
		}

		private void textBox4_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Memo = textBox4.Text.Replace("\r\n", "\n"); // 念のため。。
				foreach (ListViewItem lvItem in listView1.SelectedItems) {
					if (lvItem.Tag == currentItem) {
						lvItem.SubItems[4].Text = currentItem.Memo;
					}
				}
				configManager.Save();
			}
		}

		private void textBox6_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom1Name = textBox6.Text;
				configManager.Save();
			}
		}

		private void textBox9_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom2Name = textBox9.Text;
				configManager.Save();
			}
		}

		private void textBox11_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom3Name = textBox11.Text;
				configManager.Save();
			}
		}

		private void textBox13_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom4Name = textBox13.Text;
				configManager.Save();
			}
		}

		private void textBox7_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom1Data = textBox7.Text;
				configManager.Save();
			}
		}

		private void textBox8_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom2Data = textBox8.Text;
				configManager.Save();
			}
		}

		private void textBox10_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom3Data = textBox10.Text;
				configManager.Save();
			}
		}

		private void textBox12_TextChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom4Data = textBox12.Text;
				configManager.Save();
			}
		}

		#endregion

		Toolkit.Time.RunAt statusClearThread = new Toolkit.Time.RunAt();

		#region [C]ボタン

		private void button1_Click(object sender, EventArgs e) {
			string data = textBox1.Text;
			DoCopy(data, "ユーザー: " + data + " を貼り付けました。");
		}

		private void button2_Click(object sender, EventArgs e) {
			string data = textBox2.Text;
			DoCopy(data, "パスワードを貼り付けました。");
		}

		private void button4_Click(object sender, EventArgs e) {
			string data = textBox3.Text;
			DoCopy(data, "メール: " + data + " をクリップボードにコピーしました。");
		}

		private void button12_Click(object sender, EventArgs e) {
			string data = textBox7.Text;
			DoCopy(data, "カスタムデータ1をクリップボードにコピーしました。");
		}

		private void button15_Click(object sender, EventArgs e) {
			string data = textBox8.Text;
			DoCopy(data, "カスタムデータ2をクリップボードにコピーしました。");
		}

		private void button18_Click(object sender, EventArgs e) {
			string data = textBox10.Text;
			DoCopy(data, "カスタムデータ3をクリップボードにコピーしました。");
		}

		private void button21_Click(object sender, EventArgs e) {
			string data = textBox12.Text;
			DoCopy(data, "カスタムデータ4をクリップボードにコピーしました。");
		}

		private void DoCopy(string data, string text) {
			try {
				Clipboard.SetDataObject(data, true);
				SetStatusLabel(text);
			} catch (Exception e) {
				SetStatusLabel(e.Message);
			}
		}

		#endregion

		#region [貼]ボタン

		private void button6_Click(object sender, EventArgs e) {
			string data = textBox1.Text;
			DoPaste(data, "ユーザー: " + data + " を貼り付けました。");
		}

		private void button7_Click(object sender, EventArgs e) {
			string data = textBox2.Text;
			DoPaste(data, "パスワードを貼り付けました。");
		}

		private void button8_Click(object sender, EventArgs e) {
			string data = textBox3.Text;
			DoPaste(data, "メール: " + data + " を貼り付けました。");
		}

		private void button11_Click(object sender, EventArgs e) {
			string data = textBox7.Text;
			DoPaste(data, "カスタムデータ1: " + data + " を貼り付けました。");
		}

		private void button14_Click(object sender, EventArgs e) {
			string data = textBox8.Text;
			DoPaste(data, "カスタムデータ2: " + data + " を貼り付けました。");
		}

		private void button17_Click(object sender, EventArgs e) {
			string data = textBox10.Text;
			DoPaste(data, "カスタムデータ3: " + data + " を貼り付けました。");
		}

		private void button20_Click(object sender, EventArgs e) {
			string data = textBox12.Text;
			DoPaste(data, "カスタムデータ4: " + data + " を貼り付けました。");
		}

		/// <summary>
		/// 貼り付け処理
		/// </summary>
		private void DoPaste(string data, string text) {
			try {
				IDataObject clipboardData = Clipboard.GetDataObject();
				Clipboard.SetDataObject(data, true);
				Hide();
				SendKeys.SendWait("^v{TAB}");
				Show();
				Clipboard.SetDataObject(clipboardData, true);
				SetStatusLabel(text);
			} catch (Exception e) {
				SetStatusLabel(e.Message);
			}
		}

		#endregion

		/// <summary>
		/// [開]ボタン
		/// </summary>
		private void button5_Click(object sender, EventArgs e) {
			{
				System.Diagnostics.ProcessStartInfo info = new System.Diagnostics.ProcessStartInfo();
				info.FileName = textBox5.Text;
				info.ErrorDialog = true;
				info.ErrorDialogParentHandle = Handle;
				info.UseShellExecute = true;
				System.Diagnostics.Process.Start(info);
			}
			SetStatusLabel(textBox5.Text + " を開きました。");
		}

		#region [乱]ボタン

		private void button9_Click(object sender, EventArgs e) {
			RandomGenerateButton(textBox1, "ID");
		}

		private void button3_Click(object sender, EventArgs e) {
			RandomGenerateButton(textBox2, "パスワード");
		}

		private void button10_Click(object sender, EventArgs e) {
			RandomGenerateButton(textBox7, "カスタムデータ1");
		}

		private void button13_Click(object sender, EventArgs e) {
			RandomGenerateButton(textBox8, "カスタムデータ2");
		}

		private void button16_Click(object sender, EventArgs e) {
			RandomGenerateButton(textBox10, "カスタムデータ3");
		}

		private void button19_Click(object sender, EventArgs e) {
			RandomGenerateButton(textBox12, "カスタムデータ4");
		}

		/// <summary>
		/// ランダム生成ボタンの処理
		/// </summary>
		private void RandomGenerateButton(TextBox textBox, string name) {
			using (MakePassForm form = new MakePassForm(configManager.Config, name)) {
				if (form.ShowDialog(this) == DialogResult.OK) {
					textBox.Text = form.Pass;
				}
			}
		}

		#endregion

		#region [隠]ボタン(チェックボックス)

		private void checkBox1_CheckedChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom1Hide = checkBox1.Checked;
				configManager.Save();
			}
			DoShowHide(textBox7, checkBox1.Checked);
		}

		private void checkBox2_CheckedChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom2Hide = checkBox2.Checked;
				configManager.Save();
			}
			DoShowHide(textBox8, checkBox2.Checked);
		}

		private void checkBox3_CheckedChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom3Hide = checkBox3.Checked;
				configManager.Save();
			}
			DoShowHide(textBox10, checkBox3.Checked);
		}

		private void checkBox4_CheckedChanged(object sender, EventArgs e) {
			if (currentItem != null) {
				currentItem.Custom4Hide = checkBox4.Checked;
				configManager.Save();
			}
			DoShowHide(textBox12, checkBox4.Checked);
		}

		void DoShowHide(TextBox textBox, bool hide) {
			textBox.UseSystemPasswordChar = hide;
		}

		#endregion

		#endregion

		/// <summary>
		/// toolStripStatusLabel1.Text へ一時的に文字をセット。
		/// </summary>
		/// <param name="text"></param>
		private void SetStatusLabel(string text) {
			statusClearThread.Run = delegate() {
				try {
					if (!IsDisposed) {
						Invoke(new MethodInvoker(delegate() {
							try { toolStripStatusLabel1.Text = ""; } catch { }
						}));
					}
				} catch { }
			};
			statusClearThread.At = DateTime.Now.AddSeconds(5);
			toolStripStatusLabel1.Text = text;
		}

		#region リストビューのドラッグ処理

		private void listView1_ItemDrag(object sender, ItemDragEventArgs e) {
			listView1.DoDragDrop((ListViewItem)e.Item, DragDropEffects.Copy | DragDropEffects.Move);
		}

		private void listView1_DragEnter(object sender, DragEventArgs e) {
			if (e.Data.GetDataPresent(typeof(ListViewItem))) {
				// エフェクトの指定
				Toolkit.Forms.Utility.SetDropEffect(e, DragDropEffects.Move);
			}
		}

		private void listView1_DragOver(object sender, DragEventArgs e) {
			if (e.Data.GetDataPresent(typeof(ListViewItem))) {
				ListViewItem dstLvItem = Toolkit.Forms.Utility.GetDropTarget(listView1, e);
				if (dstLvItem != null) {
					dstLvItem.Selected = true;
				}
				// エフェクトの指定
				Toolkit.Forms.Utility.SetDropEffect(e, DragDropEffects.Move);
			}
		}

		private void listView1_DragDrop(object sender, DragEventArgs e) {
			if (e.Data.GetDataPresent(typeof(ListViewItem))) {
				ListViewItem srcLvItem = (ListViewItem)e.Data.GetData(typeof(ListViewItem));
				ListViewItem dstLvItem = Toolkit.Forms.Utility.GetDropTarget(listView1, e);

				ItemDirectory dstDir = dstLvItem == null ? null : dstLvItem.Tag as ItemDirectory;
				ItemDirectory srcDir = srcLvItem.Tag as ItemDirectory;
				Item srcItem = srcLvItem.Tag as Item;

				if (srcItem != null) { // srcがitemな場合
					Item newItem = srcItem.Clone();
					if (dstDir != null) { // ディレクトリへドロップ
						// そのディレクトリの末尾へ追加
						Add(dstDir, newItem, false);
					} else { // アイテムへドロップ / 何もないとこへドロップ
						// 並び替え
						int index = dstLvItem == null ? listView1.Items.Count : // 末尾へ
							dstLvItem.Index <= srcLvItem.Index ? dstLvItem.Index :
							dstLvItem.Index + 1;
						Insert(currentDir, newItem, index);
					}
				} else if (srcDir != null) { // srcがdirな場合
					ItemDirectory newItem = srcDir.Clone();
					if (dstDir != null) { // ディレクトリへドロップ
						// そのディレクトリの末尾へ追加
						Add(dstDir, newItem, false);
					} else { // アイテムへドロップ / 何もないとこへドロップ
						// 末尾へ
						Add(currentDir, newItem, false);
					}
				}

				if (e.Effect == DragDropEffects.Move) { // 移動な場合は削除
					if (srcItem != null) Remove(currentDir, srcItem);
					if (srcDir != null) Remove(currentDir, srcDir);
				}

				// 保存ー
				configManager.Save();
			}
		}

		#endregion
		#region ツリービューのドラッグ処理

		private void treeView1_ItemDrag(object sender, ItemDragEventArgs e) {
			treeView1.DoDragDrop((TreeNode)e.Item, DragDropEffects.Copy | DragDropEffects.Move);
		}

		private void treeView1_DragEnter(object sender, DragEventArgs e) {
			if (e.Data.GetDataPresent(typeof(TreeNode)) ||
				e.Data.GetDataPresent(typeof(ListViewItem))) {
				// エフェクトの指定
				Toolkit.Forms.Utility.SetDropEffect(e, DragDropEffects.Move);
			}
		}

		private void treeView1_DragOver(object sender, DragEventArgs e) {
			if (e.Data.GetDataPresent(typeof(TreeNode)) ||
				e.Data.GetDataPresent(typeof(ListViewItem))) {
				treeView1.SelectedNode = Toolkit.Forms.Utility.GetDropTarget(treeView1, e);
				// エフェクトの指定
				Toolkit.Forms.Utility.SetDropEffect(e, DragDropEffects.Move);
			}
		}

		private void treeView1_DragDrop(object sender, DragEventArgs e) {
			if (e.Data.GetDataPresent(typeof(TreeNode))) {
				TreeNode srcNode = (TreeNode)e.Data.GetData(typeof(TreeNode));
				OnDropToTreeView(e, (ItemDirectory)srcNode.Tag, (ItemDirectory)srcNode.Parent.Tag);
			} else if (e.Data.GetDataPresent(typeof(ListViewItem))) {
				ListViewItem srcLvItem = (ListViewItem)e.Data.GetData(typeof(ListViewItem));
				ItemDirectory srcDir = srcLvItem.Tag as ItemDirectory;
				if (srcDir != null) {
					OnDropToTreeView(e, srcDir, configManager.ItemManager.Root.FindParent(srcDir));
				} else {
					Item srcItem = srcLvItem.Tag as Item;
					if (srcItem != null) {
						OnDropToTreeView(e, srcItem, configManager.ItemManager.Root.FindParent(srcItem));
					}
				}
			}
		}
		private void OnDropToTreeView(DragEventArgs e, ItemDirectory srcData, ItemDirectory srcParentDir) {
			TreeNode dstNode = Toolkit.Forms.Utility.GetDropTarget(treeView1, e);
			ItemDirectory dstDir = (ItemDirectory)dstNode.Tag;
			if (srcData == dstDir) return; // 自分自身へのドロップは無視する

			Add(dstDir, srcData.Clone(), false);
			SetCurrentDir(dstDir);	// カレントディレクトリをドロップ先へ。

			if (e.Effect == DragDropEffects.Move) { // 移動な場合は削除
				Remove(srcParentDir, srcData);
			}

			// 保存ー
			configManager.Save();
		}
		private void OnDropToTreeView(DragEventArgs e, Item srcData, ItemDirectory srcParentDir) {
			TreeNode dstNode = Toolkit.Forms.Utility.GetDropTarget(treeView1, e);
			ItemDirectory dstDir = (ItemDirectory)dstNode.Tag;

			Add(dstDir, srcData.Clone(), false);
			SetCurrentDir(dstDir);	// カレントディレクトリをドロップ先へ。

			if (e.Effect == DragDropEffects.Move) { // 移動な場合は削除
				Remove(srcParentDir, srcData);
			}

			// 保存ー
			configManager.Save();
		}

		#endregion
	}
}