﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ぱすわど {
	public partial class MakePassForm : Form {
		Config config;
		string name;

		Random rand = new Random();
		string pass;

		public string Pass { get { return pass; } }

		public MakePassForm(Config config, string name) {
			InitializeComponent();

			this.config = config;
			this.name = name;

			Text = "ランダム" + name + "の生成";
			label1.Text = name;
		}

		/// <summary>
		/// 初期化
		/// </summary>
		private void MakePassForm_Load(object sender, EventArgs e) {
			numericUpDown1.Value = (decimal)config.MakePassLen;
			checkBox1.Checked = config.MakePassViewPass;
			checkBox2.Checked = config.MakePassUC;
			checkBox3.Checked = config.MakePassLC;
			checkBox4.Checked = config.MakePassNum;
			checkBox5.Checked = config.MakePassOther;
			textBox2.Text = config.MakePassOtherChars;
			// ↓何故か呼び出されない風味なので必要なとこだけ手動で呼び出し。
			checkBox1_CheckedChanged(this, null);
			checkBox5_CheckedChanged(this, null);
		}

		/// <summary>
		/// 設定保存ボタン
		/// </summary>
		private void button1_Click(object sender, EventArgs e) {
			config.MakePassLen = (int)numericUpDown1.Value;
			config.MakePassViewPass = checkBox1.Checked;
			config.MakePassUC = checkBox2.Checked;
			config.MakePassLC = checkBox3.Checked;
			config.MakePassNum = checkBox4.Checked;
			config.MakePassOther = checkBox5.Checked;
			config.MakePassOtherChars = textBox2.Text;
		}

		/// <summary>
		/// 採用ボタン
		/// </summary>
		private void buttonOk_Click(object sender, EventArgs e) {
			if (MessageBox.Show(this, name + "を採用します。\r\n\r\n※ 現在の値は上書きされて消えるので注意してください。", "確認",
				MessageBoxButtons.OKCancel, MessageBoxIcon.Information) == DialogResult.OK) {
				pass = textBox1.Text;
				System.Diagnostics.Debug.Assert(DialogResult == DialogResult.OK);
			} else {
				DialogResult = DialogResult.None;
			}
		}

		/// <summary>
		/// 再生成ボタン
		/// </summary>
		private void button2_Click(object sender, EventArgs e) {
			List<char> passChars = new List<char>();
			if (checkBox2.Checked) { // 大文字
				passChars.AddRange("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
			}
			if (checkBox3.Checked) { // 小文字
				passChars.AddRange("abcdefghijklmnopqrstuvwxyz");
			}
			if (checkBox4.Checked) { // 数字
				passChars.AddRange("0123456789");
			}
			if (checkBox5.Checked) { // その他
				passChars.AddRange(textBox2.Text);
			}

			if (passChars.Count <= 0) {
				// 文字が設定されてない場合。あるいは設定の反映途中な場合。
				textBox1.Text = "";
				return;
			}

			string newPass;
			while (true) {
				newPass = "";
				for (int i = 0, n = (int)numericUpDown1.Value; i < n; i++) {
					newPass += passChars[rand.Next(0, passChars.Count)];
				}

				if (checkBox2.Checked && !CharContains(newPass, "ABCDEFGHIJKLMNOPQRSTUVWXYZ")) { // 大文字
					continue; // 含まれてないなら再生成
				}
				if (checkBox3.Checked && !CharContains(newPass, "abcdefghijklmnopqrstuvwxyz")) { // 小文字
					continue; // 含まれてないなら再生成
				}
				if (checkBox4.Checked && !CharContains(newPass, "0123456789")) { // 数字
					continue; // 含まれてないなら再生成
				}
				if (checkBox5.Checked && !CharContains(newPass, textBox2.Text)) { // その他
					continue; // 含まれてないなら再生成
				}
				// 全種含まれてたらおしまい。
				break;
			}

			textBox1.Text = newPass;
		}

		/// <summary>
		/// newPassにcharsのなかの文字が１つ以上入ってたらtrue
		/// </summary>
		private static bool CharContains(string newPass, string chars) {
			foreach (char c in newPass) {
				foreach (char ch in chars) {
					if (c == ch) return true;
				}
			}
			return false;
		}

		private void checkBox1_CheckedChanged(object sender, EventArgs e) {
			textBox1.UseSystemPasswordChar = !checkBox1.Checked;
		}

		private void numericUpDown1_ValueChanged(object sender, EventArgs e) {
			button2_Click(this, null); // 再生成
		}
		private void checkBox2_CheckedChanged(object sender, EventArgs e) {
			button2_Click(this, null); // 再生成
		}
		private void checkBox3_CheckedChanged(object sender, EventArgs e) {
			button2_Click(this, null); // 再生成
		}
		private void checkBox4_CheckedChanged(object sender, EventArgs e) {
			button2_Click(this, null); // 再生成
		}
		private void checkBox5_CheckedChanged(object sender, EventArgs e) {
			button2_Click(this, null); // 再生成
			textBox2.Enabled = checkBox5.Checked;
		}
		private void textBox2_TextChanged(object sender, EventArgs e) {
			button2_Click(this, null); // 再生成
		}
	}
}